#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import annotations

import unittest

from procamora_utils.ip import IP
from procamora_utils.ping import ping


class TestIP(unittest.TestCase):

    def test_ip_up(self: TestIP):
        ip: IP = IP(ip='127.0.0.1')
        self.assertTrue(ping(ip), msg=f'{ip.get_addr()} must be up')

    def test_ip_down(self: TestIP):
        ip: IP = IP(ip='192.168.1.87')
        self.assertFalse(ping(ip), msg=f'{ip.get_addr()} must be down')

    def test_fqdn_up(self: TestIP):
        ip: IP = IP(fqdn='google.es')
        self.assertTrue(ping(ip), msg=f'{ip.get_addr()} must be up')

    def test_fqdn_down(self: TestIP):
        ip: IP = IP(fqdn='google.false')
        self.assertFalse(ping(ip), msg=f'{ip.get_addr()} must be down')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import annotations

import unittest
from pathlib import Path
from typing import NoReturn, Text, List, Dict, Any

from procamora_utils.interface_sqlite import *

# CUIDADO CON LAS MODIFICACIONES, LA COMPARACION SE HACE CON EL TAMAÑO (724)
dump_str: Text = '''BEGIN TRANSACTION;
DROP TABLE IF EXISTS "table1";
CREATE TABLE IF NOT EXISTS "table1" (
    "id"	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE,
    "value"	TEXT NOT NULL
);
DROP TABLE IF EXISTS "table2";
CREATE TABLE IF NOT EXISTS "table2" (
    "id"	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE,
    "value2"	INTEGER NOT NULL
);
INSERT INTO "table1" VALUES (1,'Python');
INSERT INTO "table1" VALUES (2,'Java');
INSERT INTO "table1" VALUES (3,'C++');
INSERT INTO "table1" VALUES (4,'Bash');
INSERT INTO "table2" VALUES (1,2.7);
INSERT INTO "table2" VALUES (2,3.5);
INSERT INTO "table2" VALUES (3,3.6);
INSERT INTO "table2" VALUES (4,3.7);
INSERT INTO "table2" VALUES (5,3.8);
COMMIT;
'''


class TestSQLite(unittest.TestCase):
    def setUp(self: TestSQLite) -> NoReturn:
        self.dump: Path = Path('resources/unittest.sql')
        self.db: Path = Path('resources/unittest.db')

        # La libreria no tiene el dump ni la bd, por lo que tiene que crearlos la primera vez
        self.dump.write_text(dump_str)
        execute_script_sqlite(self.db, self.dump.read_text())

        self.expect1: List[Dict[Text, Any]] = [{'id': 1, 'value': 'Python'}, {'id': 2, 'value': 'Java'},
                                               {'id': 3, 'value': 'C++'}, {'id': 4, 'value': 'Bash'}]
        self.expect2: List[Dict[Text, Any]] = [{'id': 1, 'value2': 2.7}, {'id': 2, 'value2': 3.5},
                                               {'id': 3, 'value2': 3.6}, {'id': 4, 'value2': 3.7},
                                               {'id': 5, 'value2': 3.8}]
        self.insert: Text = 'Perl'

    def test_exist_db(self: TestSQLite):
        """
        Compruebo que se manda una excepcion si la BD no existe
        :return:
        """
        db: Path = Path('false.db')
        query: Text = "SELECT * FROM table1"
        self.assertRaises(OSError, conection_sqlite, db, query)

    # ontengo valores de tabla1
    def test_select_table1(self: TestSQLite):
        """
        Compruebo que obtiene correctamente todos los valores de la tabla1
        :return:
        """
        query: Text = "SELECT * FROM table1"
        response_query: List[Dict[Text, Any]] = conection_sqlite(self.db, query, is_dict=True)
        self.assertEqual(len(response_query), len(self.expect1), msg="table1 missing values are not expected")

    def test_select_table2(self: TestSQLite):
        """
        Compruebo que obtiene correctamente todos los valores de la tabla2
        :return:
        """
        query: Text = "SELECT * FROM table2"
        response_query: List[Dict[Text, Any]] = conection_sqlite(self.db, query, is_dict=True)
        self.assertEqual(len(response_query), len(self.expect2), msg="table2 missing values are not expected")

    def test_insert_table1(self: TestSQLite):
        """
        Compruebo que se puede insertar correctamente un elemento
        :return:
        """
        query: Text = f"INSERT INTO table1(value) VALUES ('{self.insert}')"
        conection_sqlite(self.db, query)

        query: Text = f"SELECT * FROM table1 WHERE value LIKE '{self.insert}'"
        response_query: List[Dict[Text, Any]] = conection_sqlite(self.db, query, is_dict=True)
        self.assertEqual(len(response_query), 1, msg="insert failed")

    def test_delete_table1(self: TestSQLite):
        """

        :return:
        """
        query: Text = f"DELETE FROM table1 WHERE value LIKE 'Python'"
        conection_sqlite(self.db, query)
        # Confirmacion de que se ha borrado, volviendo a comprar que esta la tabla ya NO esta como antes
        query: Text = "SELECT * FROM table1"
        response_query: List[Dict[Text, Any]] = conection_sqlite(self.db, query, is_dict=True)
        self.assertNotEqual(len(response_query), len(self.expect1), msg="table1 missing values are not expected")

    def test_dump(self: TestSQLite):
        # FIXME Comprobar que el tamaño no cambia nunca
        response = dump_database(self.db)
        self.assertEqual(len(response), 724, msg="dump missing values are not expected")

    def tearDown(self: TestSQLite) -> None:
        """
        Tras cada test restauramos la bd
        :return:
        """
        execute_script_sqlite(self.db, self.dump.read_text())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import annotations

import logging
import re
from dataclasses import dataclass
from ipaddress import AddressValueError, ip_interface
from typing import Text, NoReturn

from procamora_utils.logger import get_logging

logger: logging = get_logging(False, 'ip')


@dataclass
class IP:
    REGEX_FQDN: Text = r"(?=^.{4,253}$)(^((?!-)[a-zA-Z0-9-]{0,62}[a-zA-Z0-9]\.)+[a-zA-Z]{2,63}$)"
    ip: Text = None
    fqdn: Text = None

    valid_ip: ip_interface = None

    def __post_init__(self: IP) -> NoReturn:
        """
        Metodo post_inicio encargado de comprobar que la informacion proporcionada es valida
        :return:
        """
        bool_fqnd: bool = False
        bool_ip: bool = False

        if self.fqdn is not None:
            if not re.search(self.REGEX_FQDN, self.fqdn):
                raise AttributeError(f'{self.fqdn} is not valid FQDN')
            else:
                bool_fqnd = True

        if self.ip is not None:
            try:
                self.valid_ip = ip_interface(self.ip)  # throw exception if invalid ip
            except ValueError as e:
                raise AttributeError(f'{e}')
            bool_ip = True

        if not bool_fqnd and not bool_ip:
            raise AttributeError('An IP address or FQDN must be introduced.')

    def get_ip(self: IP) -> Text:
        """
        Metodo para obtener la direccion IP en caso de existir, sino envia una excepcion
        :return:
        """
        if self.valid_ip is None:
            raise AttributeError('IP address is None.')
        return str(self.valid_ip.ip)

    def get_fqdn(self: IP) -> Text:
        """
        Metodo para obtener el FQDN en caso de existir, sino envia una excepcion
        :return:
        """
        if self.fqdn is None:
            raise AttributeError('FQDN is None.')
        return self.fqdn

    def get_addr(self: IP) -> Text:
        """
        Metodo para obtener el FQDN en caso de existir, sino obtiene la direccion IP
        Es recomensable usar este metodo por ser el generico.
        :return:
        """
        if self.fqdn is not None:
            return self.fqdn
        else:
            return str(self.valid_ip.ip)

    def __str__(self: IP) -> Text:
        return f'IP(ip=\'{self.valid_ip}\', fqnd=\'{self.fqdn}\')'


if __name__ == "__main__":
    # print(IP(fqdn='google.es').get_ip())
    print(IP(fqdn='google.es').get_fqdn())
    print(IP(fqdn='google.es').get_addr())

    print(IP(ip='fb7::2').get_ip())
    # print(IP(ip=IPv4Interface('192.168.4.56')).get_fqdn())
    print(IP(ip='fb7::56').get_addr())

    print(IP(ip='192.168.4.56', fqdn='google.es').get_ip())
    print(IP(ip='192.168.4.56', fqdn='google.es').get_fqdn())
    print(IP(ip='192.168.4.56', fqdn='google.es').get_addr())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging
from typing import Text

import colorlog  # https://medium.com/@galea/python-logging-example-with-color-formatting-file-handlers-6ee21d363184


def get_logging(verbose: bool, name: Text = 'procamora') -> logging:
    # Desabilita log de modulos
    # for _ in ("boto", "elasticsearch", "urllib3"):
    #    logging.getLogger(_).setLevel(logging.CRITICAL)

    log_format: Text = '%(levelname)s - %(module)s - %(funcName)s - %(message)s'

    bold_seq: Text = '\033[1m'
    colorlog_format: Text = (
        f'{bold_seq} '
        '%(log_color)s '
        f'{log_format}'
    )

    colorlog.basicConfig(format=colorlog_format)
    # logging.basicConfig(format=colorlog_format)
    log: logging = logging.getLogger(name)

    if verbose:
        log.setLevel(logging.DEBUG)
    else:
        log.setLevel(logging.INFO)

    return log



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import annotations

import unittest
from typing import Text

from procamora_utils.ip import IP


class TestIP(unittest.TestCase):

    def test_incorrect(self: TestIP):
        self.assertRaises(AttributeError, IP)

    def test_correct_fqdn(self: TestIP):
        fqdn: Text = 'test.com'
        ip: IP = IP(fqdn=fqdn)
        self.assertEqual(ip.get_addr(), fqdn, msg='not valid FQDN')

    def test_incorrect_fqdn(self: TestIP):
        fqdn: Text = 'test'
        self.assertRaises(AttributeError, IP, fqdn=fqdn)

    def test_correct_ip(self: TestIP):
        ipv4: Text = '192.168.1.1'
        ip: IP = IP(ip=ipv4)
        self.assertEqual(ip.get_addr(), str(ipv4), msg='not valid IP')

    def test_incorrect_ip(self: TestIP):
        self.assertRaises(AttributeError, IP, fqdn='192.168.1')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__all__ = ['conection_sqlite', 'execute_script_sqlite', 'dump_database']

import logging
import sqlite3
from pathlib import Path  # nueva forma de trabajar con rutas
from threading import Lock
from typing import Dict, Any, List, Union, Tuple, Optional, Text

from procamora_utils.logger import get_logging

logger: logging = get_logging(False, 'sqlite')


def conection_sqlite(database: Path, query: Text, mutex: Lock = None, is_dict: bool = False) \
        -> Union[List[Dict[Text, Any]], List[Tuple[Text, Any]], None]:
    if mutex is not None:
        mutex.acquire()  # bloqueamos acceso a db
    try:
        if database.exists():
            connection: sqlite3.Connection = sqlite3.connect(str(database))
            if is_dict:
                connection.row_factory = _dict_factory
            cursor: sqlite3.Cursor = connection.cursor()
            cursor.execute(query)

            data: Optional[List] = None
            if query.upper().startswith('SELECT'):
                data = cursor.fetchall()  # Traer los resultados de un select
            else:
                connection.commit()  # Hacer efectiva la escritura de datos

            cursor.close()
            connection.close()
            return data
        else:
            logger.critical(f'Database {database} not exits')
            raise OSError(f'Database {database} not exits')
    except sqlite3.OperationalError:
        logger.critical(f'LOCK {query}, sorry...')
    finally:
        if mutex is not None:
            mutex.release()  # liberamos mutex


def _dict_factory(cursor: sqlite3.Cursor, row: Tuple[Text]) -> Dict[Text, Text]:
    d: Dict = dict()
    for idx, col in enumerate(cursor.description):
        d[col[0]] = row[idx]
    return d


def execute_script_sqlite(database: Path, script: Text) -> None:
    connection: sqlite3.Connection = sqlite3.connect(str(database))
    cursor: sqlite3.Cursor = connection.cursor()
    cursor.executescript(script)
    connection.commit()
    cursor.close()
    connection.close()


def dump_database(database: Path) -> Optional[Text]:
    """
    Hace un dump de la base de datos y lo retorna
    :param database: ruta de la base de datos
    :return dump: volcado de la base de datos
    """
    if database.exists():
        connection: sqlite3.Connection = sqlite3.connect(str(database))
        a: Text = '\n'.join(connection.iterdump())
        return str(a)
    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging
import platform
import subprocess
from typing import NoReturn, Text, List

from procamora_utils.ip import IP
from procamora_utils.logger import get_logging

logger: logging = get_logging(False, 'ping')


def ping(hostname: IP) -> bool:
    # Posibles opciones: 'Linux', 'Darwin', 'Java', 'Windows'
    command: Text
    if platform.uname()[0] == 'Linux':
        command = f'ping -c 1 {hostname.get_addr()}'
    elif platform.uname()[0] == 'Windows':
        command = f'ping -n 1 {hostname.get_addr()}'
    else:
        logger.error(f'{platform.uname()[0]} operating system temporarily not supported')
        raise OSError(f'{platform.uname()[0]} operating system temporarily not supported')

    logger.debug(command)

    a: subprocess.CompletedProcess = subprocess.run(command.split(' '), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    # 0 -> Ejecucion correcta (online)
    # 1 -> Error (ofline)
    response: bool = False
    if a.returncode == 0:
        response = True

    # and then check the response...
    if response:
        logger.debug(f'{hostname.get_addr()} is up!!')
    else:
        logger.debug(f'{hostname.get_addr()} is down :(')
    return response


def main() -> NoReturn:
    ips: List[IP] = [IP(ip="127.0.0.1"), IP(ip="192.168.0.103"), IP(fqdn="google.es"), IP(fqdn="google.false")]

    for ip in ips:
        texto: bool = ping(ip)
        if texto:
            logger.info("{} up".format(ip.get_addr()))
        else:
            logger.info("{} down".format(ip.get_addr()))


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from procamora_utils.interface_sqlite import *
from procamora_utils.ip import IP
from procamora_utils.logger import get_logging
from procamora_utils.ping import ping



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pathlib import Path

from setuptools import setup

# read the contents of your README file
this_directory = Path(__file__).parent.resolve()
long_description = Path(this_directory, 'README.md').read_text()

version = '1.2'

setup(
    name='procamora-utils',  # How you named your package folder (MyLib)
    packages=['procamora_utils'],  # Chose the same as "name"
    version=version,  # Start with a small number and increase it with every change you make
    license='gpl-3.0',  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description='This repository contains a set of libraries that i use in other projects',
    # Give a short description about your library
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='procamora',  # Type in your name
    author_email='pablojoserocamora@gmail.com',  # Type in your E-Mail
    url='https://github.com/procamora/python3-utils',  # Provide either the link to your github or to your website
    download_url=f'https://github.com/procamora/python3-utils/archive/{version}.tar.gz',  # I explain this later on
    keywords=['sqlite', 'sql', 'sqlite3', 'network', 'fqdn', 'ip', 'ipv4', 'ipv6', 'icmp', 'ping', 'logging', 'color',
              'log'],
    # Keywords that define your package best
    install_requires=[  # I get to this in a second
        'colorlog',
    ],
    classifiers=[  # https://pypi.org/classifiers/
        'Development Status :: 4 - Beta',
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Networking',
        'Topic :: System :: Logging',
        'Topic :: Database',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',  # Again, pick a license
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    # py_modules=["interface_sqlite"],  # Required
    python_requires='<4, >=3.5',
)
